@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/CGAL_ipelets/include/CGAL/CGAL_Ipelet_base_v6.h $
+// $Id: CGAL_Ipelet_base_v6.h 01af331 2018-03-14T15:59:38+01:00 Sébastien Loriot
+// SPDX-License-Identifier: LGPL-3.0+
 // 
 //
 // Author(s)     : Sebastien Loriot, Sylvain Pion
@@ -29,7 +30,6 @@ typedef unsigned int uint;
 
 #include <ipelib.h>
 #include <CGAL/Polygon_2.h>
-#include <CGAL/Weighted_point.h>
 #include <CGAL/iterator.h>
 #include <CGAL/Triangulation_2.h>
 #include <CGAL/grabbers.h>
@@ -57,7 +57,7 @@ namespace CGAL{
     //typedefs
     typedef typename Kernel::FT                                               FT;
     typedef typename CGAL::Point_2<Kernel>                                    Point_2;
-    typedef typename CGAL::Weighted_point<Point_2,FT>                         Weighted_point_2;
+    typedef typename CGAL::Weighted_point_2<Kernel>                           Weighted_point_2;
     typedef typename Kernel::Segment_2                                        Segment_2;
     typedef typename Kernel::Ray_2                                            Ray_2;
     typedef typename Kernel::Line_2                                           Line_2;
@@ -79,7 +79,7 @@ namespace CGAL{
     int IpelibVersion() const { return IPELIB_VERSION; }
     int NumFunctions() const { return nbf; }
     virtual const char *Label() const{ return &Name[0]; }
-    const char *About() const {return "http://www.cgal.org";};
+    const char *About() const {return "https://www.cgal.org";};
     virtual const char *SubLabel(int function) const {return &SubLab[function][0];};
     virtual const char *HelpMsg(int function) const{return &HMsg[function][0];};
     void Run (int i, IpePage *page, IpeletHelper *helper) {
@@ -89,7 +89,7 @@ namespace CGAL{
         protected_run(i);
       }
       catch(...){
-        helper->MessageBox("Error : Save your page in a file and submit it to \n http://www.cgal.org/bug_report.html","OK",NULL,NULL);
+        helper->MessageBox("Error : Save your page in a file and submit it to \n https://www.cgal.org/bug_report.html","OK",NULL,NULL);
       }
     };
 
