@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Partition_2/include/CGAL/Partition_2/Indirect_not_less_yx_2.h $
+// $Id: Indirect_not_less_yx_2.h f2391a3 2017-12-19T16:29:53+01:00 Laurent Rineau
+// SPDX-License-Identifier: GPL-3.0+
 // 
 //
 // Author(s)     : Susan Hert <hert@mpi-sb.mpg.de>
@@ -29,7 +30,8 @@ namespace CGAL {
 template <class Traits>
 class Indirect_not_less_yx_2 
 {
-   public:
+public:
+     typedef typename Traits::Point_2 Point_2;
      typedef typename Traits::Less_yx_2     Less_yx_2;
 
      Indirect_not_less_yx_2(const Traits& traits) : 
@@ -38,7 +40,7 @@ class Indirect_not_less_yx_2
      template <class Iterator>
      bool 
      operator()( const Iterator& p, const Iterator& q) const
-     { return less_yx_2( *q, *p); }
+     { return less_yx_2( Point_2(*q), Point_2(*p)); }
 
    private:
      Less_yx_2 less_yx_2;
