@@ -12,10 +12,20 @@
 //
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+//
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Surface_mesh_segmentation/include/CGAL/mesh_segmentation.h $
+// $Id: mesh_segmentation.h 2f9408f 2018-09-04T12:01:27+02:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0+
+//
+// Author(s)     : Ilker O. Yaz
 
 
 #define CGAL_SURFACE_MESH_SEGMENTATION_MESH_SEGMENTATION_H
 
+#include <CGAL/license/Surface_mesh_segmentation.h>
+
+#include <CGAL/disable_warnings.h>
+
 /**
  * @file mesh_segmentation.h
  * @brief The API which contains free template functions for SDF computation and mesh segmentation.
@@ -23,6 +33,7 @@
 #include <CGAL/internal/Surface_mesh_segmentation/Surface_mesh_segmentation.h>
 #include <CGAL/boost/graph/helpers.h>
 #include <boost/config.hpp>
+#include <CGAL/Kernel/global_functions_3.h>
 
 namespace CGAL
 {
@@ -58,7 +69,7 @@ sdf_values( const TriangleMesh& triangle_mesh,
 /// @endcond
 
 /*!
- * \ingroup PkgSurfaceSegmentation
+ * \ingroup PkgSurfaceMeshSegmentationRef
  * @brief Function computing the Shape Diameter Function over a surface mesh.
  *
  * This function implements the Shape Diameter Function (SDF) as described in \cgalCite{shapira2008consistent}.
@@ -106,7 +117,7 @@ sdf_values( const TriangleMesh& triangle_mesh,
 
 
 /*!
- * \ingroup PkgSurfaceSegmentation
+ * \ingroup PkgSurfaceMeshSegmentationRef
  * @brief Function post-processing raw SDF values computed per facet.
  *
  * Post-processing steps applied :
@@ -141,7 +152,7 @@ sdf_values_postprocessing(const TriangleMesh& triangle_mesh,
 
 
 /*!
- * \ingroup PkgSurfaceSegmentation
+ * \ingroup PkgSurfaceMeshSegmentationRef
  * @brief Function computing the segmentation of a surface mesh given an SDF value per facet.
  *
  * This function fills a property map which associates a segment-id (in [0, number of segments -1])
@@ -241,7 +252,7 @@ segmentation_via_sdf_values(const TriangleMesh& triangle_mesh,
 
 
 /*!
- * \ingroup PkgSurfaceSegmentation
+ * \ingroup PkgSurfaceMeshSegmentationRef
  * @brief Function computing the segmentation of a surface mesh.
  *
  * This function is equivalent to calling the functions `CGAL::sdf_values()` and
@@ -489,4 +500,6 @@ segmentation_via_sdf_values(const TriangleMesh& triangle_mesh,
 
 }//namespace CGAL
 
+#include <CGAL/enable_warnings.h>
+
 #endif // CGAL_SURFACE_MESH_SEGMENTATION_MESH_SEGMENTATION_H //
